#!/bin/bash
#
# Install software while it is running from CD into home directory - useful in combination with persistant home or saved config.
# 
# "Reloaded"
#
# GPL
#
# Author: Fabian Franz <knx-live-inst@fabian-franz.de>
#

# Options

KEEP_DL_FILES="yes"
INTERACTIVE="yes"
STRICT_MD5SUM="no"

# Constants

export DESTDIR="$HOME/.dist"
TMPDIR="/tmp/knx-live-inst.$$"
[ -d "$(pwd)/modules" ] && MODPATH=$(pwd) || MODPATH=/usr/share/knx-live-inst
MODULES=$MODPATH/modules/
FILES="files"
EULA="eula.sh"
INSTALL="install.sh"
EXIT="exit.sh"
WGET="wget"
APTGET="$DESTDIR/bin/apt-get"

#
# needed functions
#

function deb-unpack()
{
  ar p $1 data.tar.gz | tar xz
}

function apt-unpack()
{
  (cd $1;
  for i in *.deb
  do
    dpkg -x $i .
  done)
  [ $? -ne 0 ] && return 2
}

function rpm-unpack()
{
  rpm2cpio $1 | cpio --extract --make-directories
}

function tgz-unpack()
{
  tar xzf $1
}

function tbz-unpack()
{
  tar xzj $1
}

# knx-unpack filename location type md5sum
function knx-unpack()
{
  [ ! -f $1 -a ! "$3"="apt" ] && return 2
  case "$3" in
  	deb) deb-unpack $1 ;;
	apt) apt-unpack $1 ;; 
  	rpm) rpm-unpack $1 ;;
  	tgz) tgz-unpack $1 ;;
  	tbz) tbz-unpack $1 ;;
	*) 
	  echo $"Unknown Package format" 
	  false
	;;
  esac
}

function setup_destdir()
{
  # setup $HOME/.bashrc
  
  mkdir -p $DESTDIR/bin
  mkdir -p $DESTDIR/lib
  mkdir -p $DESTDIR/man

  cp $HOME/.bashrc $HOME/.bashrc.templ.$$
  cat $HOME/.bashrc.templ.$$ | grep -v "DESTDIR=" | grep -v "export MANPATH=\$DESTDIR/man" | grep -v "export PATH=\$DESTDIR/bin/" | grep -v "export LD_LIBRARY_PATH=\$DESTDIR/lib" > $HOME/.bashrc
  echo "DESTDIR=$DESTDIR" >> $HOME/.bashrc
  echo "export MANPATH=\$DESTDIR/man/:\$MANPATH" >> $HOME/.bashrc
  echo "export PATH=\$DESTDIR/bin/:\$PATH" >> $HOME/.bashrc
  echo "export LD_LIBRARY_PATH=\$DESTDIR/lib:\$LD_LIBRARY_PATH" >> $HOME/.bashrc
  rm -f $HOME/.bashrc.templ.$$
}

#
# Main Program 
#

mkdir -p $TMPDIR
cd $TMPDIR

function clean_exit
{
  cd ~
  rm -rf $TMPDIR
}

trap "clean_exit" EXIT

export TEXTDOMAINDIR="/usr/share/locale"
export TEXTDOMAIN=knx-live-inst

# Functions needed

# Commandline parsing
while [ $# -ne 0 ];
do
  if [ "$1" = "-ni" ];
  then
    INTERACTIVE="no"
  fi
  
  if [ "$1" = "-i" ];
  then
    INTERACTIVE="yes"
  fi

  if [ "$1" = "--help" ];
  then
    echo $"Syntax: $0 -ni -s <module-name>" # TODO!
    exit 1
  fi

  if [ "$1" = "-s" ];
  then
    MODULE_NAME=$2
  fi

  shift # next parameter
done

function abort
{

  if [ $INTERACTIVE="yes" ]
  then
    echo $1>$TMPDIR/error$$
    return 1
  fi
  echo $1
  exit $2
}

function compare_md5sum
{
  [ "$2"="none" ] && return
  MD5SUM=$(md5sum $1 | awk '{ print $1 }')
  if [ "$MD5SUM" != "$2" ]
  then 
    update_status $"Warning: MD5DUM does not match for file $1!"
    [ $STRICT_MDSUM = "yes" ] && return 1
  fi
}

# Format of Files order + package_name
# So package basename location type md5sum
function get_file()
{
  cd $TMPDIR
  if [ $4 = "apt" ]
  then
    if [ ! -x "$DESTDIR/bin/apt-get" ]
    then
      abort $"Fatal error: apt-get was not yet installed. Please do this, before you try to install this package." 1
      return 1
    fi
    rm -f $DESTDIR/var/cache/apt/archives/*.deb
    cp $MODULES/$1/*.deb $DESTDIR/var/cache/apt/archives/ 2>/dev/null
    $APTGET update

    RELEASE=
    if [ "$3" = "unstable" -o "$3" = "testing" ]
    then
      RELEASE="-t $3"
    fi
    
    $APTGET $RELEASE -y -d install $1
    mkdir -p $TMPDIR/$1
    if [ "$KEEP_DL_FILES" = "yes" ]
    then
      cp -f $DESTDIR/var/cache/apt/archives/*.deb $MODULES/$1/
    fi
    mv $DESTDIR/var/cache/apt/archives/*.deb $TMPDIR/$1
    echo "$TMPDIR/$1"
    return 0
  fi
  [ -f $MODULES/$1/$2 ] && cp -f $MODULES/$1/$2 $TMPDIR
  [ -f $MODULES/$1/$2 ] || (rm -f $2; $WGET $3)
  [ -n "$5" ] && compare_md5sum $2 $5
  [ $? -ne 0 ] && return 1
  [ "$KEEP_DL_FILES" = "yes" -a -f $2 ] && cp -f $2 $MODULES/$1/$2
}

function install_menu
{
  [ -f "$1/$i/description" ] && DESC=$(eval echo \$\""$(cat $1/$i/description | sed 's/"//g')"\") || DESC=$"Software"
  PROG=$(cat $MODULES/$1/menu)
  NAMESOFT=$"Extra Software"

  mkdir -p $HOME/.kde/share/applnk/Knoppix/$NAMESOFT
  cat >$HOME/.kde/share/applnk/Knoppix/$PROG.desktop << EOF
[Desktop Entry]
Exec=$DESTDIR/bin/$PROG
Name=$DESC
Icon=filesave.png
Type=Application
EOF
}

function update_status
{
  [ $INTERACTIVE = "yes" ] && echo -e "XXX\n$1\nXXX\n$2" || echo "$1"
}

function non_interactive()
{
if [ "$#" -lt "1" ]; then
  echo $"Usage: $(basename $0) -ni -s <module-name>"
  echo $"Availabe are:"
  ls $MODULES | tr ' ' \n
  exit 1
fi 

if [ ! -f "$MODULES/$1/$FILES" ];
then
  echo $"Fatal: Package not found"
  echo $"Availabe are:"
  ls $MODULES | tr ' ' \n
  exit 1
fi

update_status $"OK, Installing $1..." 20
setup_destdir

NUM=$(cat $MODULES/$1/files | egrep -v "^#" | wc -l | sed 's/ //g')
update_status $"Getting $NUM files..." 40
I="1"
while [ "$I" -le "$NUM" ];
do
  file=$(cat $MODULES/$1/files | egrep -v "^#" | tail +$I | head -n 1) 
  get_file $1 $file || return 1
  I=$(($I+1))
done

update_status $"Unpacking $NUM files..." 60
I="1"
while [ "$I" -le "$NUM" ];
do
  file=$(cat $MODULES/$1/files | egrep -v "^#" | tail +$I | head -n 1) 
  knx-unpack $file
  if [ $? -eq 2 ]
  then
    set -- $file
    abort $"Fatal error: File $1 not found!\nPerhaps your internet connection is not available at the moment?" 1
    return 1
  fi
  I=$(($I+1))
done

cd $TMPDIR
update_status $"Installing program..." 80
. $MODULES/$1/install.sh
[ -f $MODULES/$1/menu ] && install_menu $1
update_status $"All done" 100
}

if [ $INTERACTIVE = "no" ]
then
  non_interactive $MODULE_NAME
  exit 0
fi

## Interactive Part

function setup_dialog()
{
  # Set up dialog
  XDIALOG_HIGH_DIALOG_COMPAT=1
  export XDIALOG_HIGH_DIALOG_COMPAT

  TMP="$TMPDIR/dialog"

  DIALOG="dialog"
  [ -n "$DISPLAY" ] && [ -x /usr/bin/Xdialog ] && DIALOG="Xdialog"
}

setup_dialog
WGET="xterm -e wget"
APTGET="xterm -e $APTGET"
BT=$"Knoppix-Live Installer"

function exit_proc
{
  unset IFS
  $DIALOG --backtitle "$BT" --title $"Really quit?" --yesno $"Do you really want to quit?" 0 0
  RC=$?
  [ $RC -ne 255 ] && exit 0
  RC=0
}

function get_software
{ 
  for i in $(ls $1)
  do
    if [ -d $1/$i ]
    then
      [ -f "$1/$i/deactivated" ] && continue
      [ -f "$1/$i/description" ] && DESC=$(eval echo \$\""$(cat $1/$i/description | sed 's/"//g')"\") || DESC=$"Software"
      echo -en "$i|$DESC|off|"
    fi
  done
}

function choose_soft
{
  SOFTWARE=$(get_software $MODULES)
  IFS="|"
  $DIALOG --backtitle "$BT" --title $"Choose Software" --radiolist $"Choose the software you wish to install." 0 0 6 $SOFTWARE 2>$TMP
  RC=$?
  unset IFS
  MODULE_NAME=$(cat $TMP)
  return $RC
}

function show_eula
{
 [ -f $MODULES/$MODULE_NAME/$EULA ] || return 0

 notfound=0
 for i in $(cat $MODULES/$MODULE_NAME/$FILES | grep -v "#" | awk '{ print $1 }')
 do
   [ -f $MODULES/$MODULE_NAME/$i ] || notfound=1
 done
 if [ $notfound -ne 0 ]
 then
   . $MODULES/$MODULE_NAME/$EULA
 fi
}

function show_module_exit
{
  [ -f $MODULES/$MODULE_NAME/$EXIT ] || return 0 
  . $MODULES/$MODULE_NAME/$EXIT
}

function main()
{
  $DIALOG --backtitle "$BT" --title $"Disclaimer" --msgbox $"This script is able to install certain software, \nwhile knoppix is running from CD. Note that this is still an experimental feature.\n No warranty is given...\nThe software will be downloaded from the internet, if its not found locally.
\n\nAuthor: Fabian Franz <knoppix-live@fabian-franz.de>" 0 0

  choose_soft || exit 1
  
  show_eula || exit 1
  echo>$TMPDIR/error$$
  [ $RC = "0" ] && non_interactive $MODULE_NAME | $DIALOG --backtitle "$BT" --title $"Installing $MODULE_NAME ..." --gauge $"Installing" 10 60
  ERROR=$(cat $TMPDIR/error$$)
  if [ -n "$ERROR" ]
  then
    $DIALOG --backtitle "$BT" --title $"Fatal error" --msgbox "$ERROR" 0 0
    return 1
  fi
  show_module_exit # needed for example for nvidia-drivers
}

if [ ! -x "/KNOPPIX/bin/ash" ]
then
  $DIALOG --backtitle "$BT" --title $"Fatal error. Not in live environment." --msgbox $"This script should only be run from the knoppix-live-environment.\nIf you want to install certain programs, please take a look at:\n\n http://kano.mipooh.net\n\nand use debian-tools like apt-get." 0 0
  exit 5
fi

main || exit 1
